--------------------------------------------------------------------------------
-- Copyright (c) 2001 by Trenz Electronic.
-- Duenner Kirchweg 77, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2S Application Note: SoC with XR16 RISC CPU.
-- File:        Glue.vhd
-- Description: Glue logic.
-- History:     2001oct05, FB, Created.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity Glue is
	port(
		clk :      in    STD_LOGIC;                     -- global clock
		rst :      in    STD_LOGIC;                     -- global async reset
		-- XR16 processor
		adx_nxt :  in    STD_LOGIC_VECTOR(15 downto 0); -- address of next memory access
		insn :     out   STD_LOGIC_VECTOR(15 downto 0); -- new instruction word
--		d :        inout STD_LOGIC_VECTOR(15 downto 0); -- on-chip data bus
		di :       in    STD_LOGIC_VECTOR(15 downto 0); -- on-chip data bus
		do :       out   STD_LOGIC_VECTOR(15 downto 0); -- on-chip data bus
		dbus_nxt : in    STD_LOGIC;                     -- next access uses on-chip data bus
		mem_ce :   in    STD_LOGIC;                     -- memory access clock enable
		read_nxt : in    STD_LOGIC;                     -- next access is read
		word_nxt : in    STD_LOGIC;                     -- next access is word wide
		rdy :      out   STD_LOGIC;                     -- current memory access is ready
		ld_t :     out   STD_LOGIC;                     -- active low store data LSB output enable
		ud_t :     out   STD_LOGIC;                     -- active low store data MSB output enable
		udlt_t :   out   STD_LOGIC;                     -- active low store data MSB->LSB output en
		dma :      in    STD_LOGIC;                     -- current access is a DMA transfer
		dma_req :  out   STD_LOGIC;                     -- DMA request
		zero_dma : out   STD_LOGIC;                     -- zero DMA counter request
		int_req :  out   STD_LOGIC;                     -- interrupt request
		-- firmware ROM & user RAM
		iadx :     out   STD_LOGIC_VECTOR(11 downto 0); -- address
		irdd :     in    STD_LOGIC_VECTOR(15 downto 0); -- read (code & data)
		iwrd :     out   STD_LOGIC_VECTOR(15 downto 0); -- write (data)
		iwrl :     out   STD_LOGIC;                     -- write enable (low)
		iwrh :     out   STD_LOGIC;                     -- write enable (high)
		-- TE-BL & VGA
		led :      out   STD_LOGIC_VECTOR(7 downto 0);  -- LEDs
		disp :     out   STD_LOGIC_VECTOR(15 downto 0); -- 7-segment displays
		sw :       in    STD_LOGIC_VECTOR(7 downto 0);  -- switches
		pb :       in    STD_LOGIC_VECTOR(7 downto 0);  -- push buttons
		vadx :     out   STD_LOGIC_VECTOR(10 downto 0); -- VGA address
		vwrd :     out   STD_LOGIC_VECTOR(7 downto 0);  -- VGA char
		vwren :    out   STD_LOGIC                      -- VGA write enable
		);
end Glue;

--------------------------------------------------------------------------------
architecture BHV of Glue is
	signal adx:      STD_LOGIC_VECTOR(15 downto 0);   -- rc(adx_nxt)
	signal word:     STD_LOGIC;                       -- rc(word_nxt)
	signal read:     STD_LOGIC;                       -- rc(read_nxt)
	signal dbus:     STD_LOGIC;                       -- rc(dbus_nxt)
	signal rdd:      STD_LOGIC_VECTOR(15 downto 0);   -- read data mux
	signal rdd8:     STD_LOGIC_VECTOR(15 downto 0);   -- dito with 8-bit access
	signal rddio:    STD_LOGIC_VECTOR(15 downto 0);   -- read data, i/o register
	signal sel_vga:  STD_LOGIC;                       -- select vga
	signal sel_led:  STD_LOGIC;                       -- select led
	signal sel_disp: STD_LOGIC;                       -- select 7-segment
	signal sel_pb:   STD_LOGIC;                       -- select push button
	signal sel_sw:   STD_LOGIC;                       -- select switches
begin
	
	------------------------------------
	-- derive 'current' signals from 'next' signals
	process(clk, rst)
	begin
		if rst= '1' then
			adx <= x"0000";
			word<= '0';
			read<= '0';
			dbus<= '0';
		elsif rising_edge(clk) then
			adx <= adx_nxt;
			word<= word_nxt;
			read<= read_nxt;
			dbus<= dbus_nxt;
		end if;
	end process;
	
	------------------------------------
	-- firmware ROM & user RAM
	--     note: memory is big-endian:
	--           (short*)(p) == 256*(char*)(p) + (char*)(p+1)
	
	-- instruction read: address and data
	iadx<= adx_nxt(11 downto 0);
	insn<= irdd;
	
	-- data read: from RAM if lower 32kB, peripherals if upper 32kB
	rdd<= rddio when adx(15)= '1' else irdd;
	
	-- data read: switch byte lanes for byte/ word access
	rdd8(15 downto 8)<= rdd(15 downto 8);
	rdd8(7  downto 0)<= rdd(7  downto 0) when word= '1' or adx(0)= '1'   -- lo on word or odd
	else                rdd(15 downto 8);                                -- hi on even
	
	-- data read: drive data onto result bus
	do(15 downto 8)<= rdd8(15 downto 8) when dbus= '1' and read= '1' and word= '1' else "ZZZZZZZZ";
	do(7  downto 0)<= rdd8(7  downto 0) when dbus= '1' and read= '1'               else "ZZZZZZZZ";
	
	-- data write: switch byte lanes for byte/ word access
	iwrd(15 downto 8)<= di(15 downto 8) when word_nxt= '1' else di(7 downto 0);
	iwrd(7 downto 0) <= di(7 downto 0);
	
	-- data write: enable if lower 32kB
	iwrl<= not(read_nxt) when adx_nxt(15)= '0' and (word_nxt= '1' or adx_nxt(0)= '1') else '0'; -- word / odd
	iwrh<= not(read_nxt) when adx_nxt(15)= '0' and (word_nxt= '1' or adx_nxt(0)= '0') else '0'; -- word / even
	
	------------------------------------
	-- control bus
	rdy<= '1';           -- no wait states
	ld_t<= read_nxt;     -- result mux 7..0
	ud_t<= read_nxt;     -- result mux 15..8
	udlt_t<= '1';        -- never redirect 15..8 to 7..0 as mem is 16-bit wide
	
	dma_req <= '0';      -- no dma requests
	zero_dma<= '0';      -- dma reset
	int_req <= '0';      -- no interrupt requests
	
	------------------------------------
	-- peripheral select signals
	sel_vga <= '1' when adx_nxt(15 downto 12)= x"8" else '0';
	sel_led <= '1' when adx_nxt(15 downto 12)= x"9" else '0';
	sel_disp<= '1' when adx_nxt(15 downto 12)= x"A" else '0';
	sel_pb  <= '1' when adx_nxt(15 downto 12)= x"B" else '0';
	sel_sw  <= '1' when adx_nxt(15 downto 12)= x"C" else '0';
	
	------------------------------------
	-- VGA: 8-bit write only
	vadx <= adx_nxt(vadx'range);
	vwrd <= di(vwrd'range);
	vwren<= sel_vga and not(read_nxt);
	
	------------------------------------
	-- led: 8-bit write only
	-- 7-segment: 16-bit write only
	process(clk, rst)
	begin
		if rst= '1' then
			led <= (others=> '0');
			disp<= (others=> '0');
		elsif rising_edge(clk) then
			if sel_led=  '1' and read_nxt= '0' then led <= di(7  downto 0); end if;
			if sel_disp= '1' and read_nxt= '0' then disp<= di(15 downto 0); end if;
		end if;
	end process;
	
	------------------------------------
	-- push buttons & switches: 8-bit read only
	process(clk, rst)
	begin
		if rst= '1' then
			rddio<= x"0000";
		elsif rising_edge(clk) then
			if sel_pb= '1' then rddio<= pb & x"00"; end if; -- 8-bit, big-endian!
			if sel_sw= '1' then rddio<= sw & x"00"; end if; -- 8-bit, big endian!
		end if;
	end process;
	
end BHV;
--------------------------------------------------------------------------------
-- end of file
